function [ CbEV, EV, TransProb, thetaProbs, RC]  = DefVar(X)

global x N M M1 M2 beta;
%
%####  DECLARE STRUCTURAL PARAMETERS TO BE ESTIMATED #####
%####  Parameters for cost function  #####
%  c(x, thetaCost) = thetaCost[1]*x + thetaCost[2]*x^2
%var thetaCost {1..2} >= 0; 
thetaCost = X(1:2);

%
%####  Parameters and definition of transition process  #####
%  thetaProbs defines Markov chain
%var thetaProbs {1..3} >= 0.00001; 
thetaProbs = X(3:5);

%
%  Y is the vector of elements in transition rule
TransProb = ((1:M)'<=M1)*thetaProbs(1)/M1 + and( (1:M)'>M1 , (1:M)'<=M2) *thetaProbs(2)/(M2-M1) + ( (1:M)'>M2 )*thetaProbs(3)/(M-M2); 


%#####  Scrap value parameter  #####
%var RC >= 0;
RC = X(6);
%##### END OF STRUCTURAL VARIABLES  #####

%####  DECLARE EQUILIBRIUM CONSTRAINT VARIABLES #####
% The NLP approach requires us to solve equilibrium constraint variables
EV = X(7:(7-1+N));
%####  END OF EQUILIBRIUM CONSTRAINT VARIABLES  #####
%
%####  DECLARE AUXILIARY VARIABLES  ######
%  Define auxiliary variables to economize on expressions	
%  Create Cost variable to represent the cost function; 
%  Cost[i] is the cost of regular maintenance at x[i].

Cost  = thetaCost(1)*x + thetaCost(2)*(x.*x);

%  Let CbEV[i] represent - Cost[i] + beta*EV[i]; 
%  this is the expected payoff at x[i] if regular maintenance is chosen
CbEV = - Cost + beta*EV;                     


end