diary mle_su_4.out;

%####  SET UP THE PROBLEM  #####
%
%  Define the state space used in the dynamic programming part
global N;
N=201; % number of states used in dynamic programming approximation
xmin=0;
xmax=100;
%  x(i) denotes state i; the set of states is a uniform grid on the interval [xmin, xmax]
global x;
x= ((1:N)'-1)*(xmax-xmin)/(N-1)+xmin;

% define discount factor, beta. 
global beta;  

beta = 0.95; 

%  Define and process the data

load BusSimData1;

nT=length(Xt);

%  The dynamic programming model in the estimation lives on a discrete state
%  Binning process: assign true mileage Xt[t] to the closest state in X
global xt
xt = ceil(Xt/(xmax-xmin)*(N-1)+0.5);

%
%  Define the Markov chain representing the changes in mileage on the x[i] grid.
%
%  The state increases by some amount in [0,JumpMax] where 
%  JumpMax is the maximum increase in mileage in one period
JumpRatio = 0.028;
JumpMax = (xmax-xmin) * JumpRatio;

global M M1 M2;
%
%  We assume that the jumps are independent of the current state
%  and its transition process has three pieces, each a uniform distribution    
%  Define 1st break point for stepwise uniform distribution in mileage increase
M1 = ceil(1/4*JumpMax/(xmax-xmin)*(N-1)+0.5);   
%  Define 2nd break point for stepwise uniform distribution in mileage increase
M2 = ceil(3/4*JumpMax/(xmax-xmin)*(N-1)+0.5);   
%Define end point for stepwise uniform distribution in mileage increase
M = ceil(JumpMax/(xmax-xmin)*(N-1)+0.5);       

%Define gradients
 global  PayoffDiffPrime TransProbPrime CbEVPrime;
        PayoffDiffPrime = zeros(6+N,length(x));
        PayoffDiffPrime(1,:)=x'-repmat(x(1),1,length(x));
        PayoffDiffPrime(2,:)=x'.^2-repmat(x(1)^2,1,length(x));
        PayoffDiffPrime(6,:)=-1;
        PayoffDiffPrime(7,:)= beta;
        
        PayoffDiffPrime= -beta*[zeros(6,N); eye(N)] + PayoffDiffPrime;
        
        CbEVPrime = zeros(length(x),6+N);
        CbEVPrime(:,1)=-x;
        CbEVPrime(:,2)=-x.^2;
        CbEVPrime= beta*[zeros(N,6) eye(N)] + CbEVPrime;
        CbEVPrime =  [ CbEVPrime; repmat(CbEVPrime(length(x),:),M,1)];
        
        TransProbPrime = zeros(6+N,M);
        TransProbPrime(3,1:M1) = 1/M1;
        TransProbPrime(4,(M1+1):M2) = 1/(M2-M1);
        TransProbPrime(5,(M2+1):M) = 1/(M-M2);    


%Define upper and lower bound for X
%thetaCost

lb = zeros(N+6,1);
ub = zeros(N+6,1);

lb(1:2)=0;
ub(1:2)=inf;
%ub(1:2)=1;

%thetaProbs
lb(3:5)=0.00001;
ub(3:5)=1;
%RC
lb(6)=0;
%ub(6)=2;
ub(6)=inf;
%EV
%#  Put bound on EV; this should not bind, but is a cautionary step to help keep algorithm within bounds
%lb(7:(7-1+N))=0;
lb(7:(7-1+N))=-inf;
ub(7:(7-1+N))=50;

% X is the solution vector
X = zeros(6+N,1);

%The probability parameters in transition process must add to one
%Linear constraint : sum proba = 1
Aeq = zeros(1, length(X));
Aeq(3:5)=1;
beq=1;

%-------------------
%Solution from AMPL:
X(3)=0.364364;
X(4)=0.599599;
X(5)=0.036036;
X(6)=1.139482;
X(1)=0.054644;
X(2)=0.001483;
%End solution from AMPL
%----------------------

Xsol = [];
Tsol = [];
fvalsol = [];
flagsol = [];
%---------------------------------------------------------
%---fmincon with gradient and without gradient sparsity
%---(fmincon does not take gradient sparsity into account)
%---fmincon stops before TolFun<1E-15
%---------------------------------------------------------

%Initial estimates
X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;



% fminconopts = optimset('DerivativeCheck','off','Display','iter',...
%            'GradConstr','on','GradObj','on','TolCon',1E-15,'TolFun',1E-15,'TolX',1E-15);   

fminconopts = optimset('DerivativeCheck','on','Display','iter',...
          'GradConstr','on','GradObj','on','TolCon',1E-6,'TolFun',1E-6,'TolX',1E-6);      
      

tic
t = cputime;
[X fval flag] = fmincon(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,fminconopts);
tsol = cputime - t;
toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];

%--------------------------------------------------
%---knitro with gradient and with gradient sparsity
%---(for comparison purpose with AMPL knitro)
%--------------------------------------------------
    
%Prepare the sparsity pattern of constraints gradient

    indices = repmat((1:N)',1,M)+repmat((1:M),N,1)-1;

    Rprime = zeros(1,6+N);
    Rprime(6)=-1;
    Rprime(7)=beta;
    
    %derivative of log(s)

    d1 = ((CbEVPrime.*repmat(ones(N+M,1),1,length(X)) + 1*repmat(Rprime,N+M,1)))./(repmat(ones(N+M,1)+1,1,length(X)));
       
    sum1 =  reshape(sum(reshape(d1(indices',:) .* repmat(repmat(ones(M,1),N,1),1,length(X)),M,N, length(X) )),N,length(X)); 
    
    sum2 = ones(N,M)*TransProbPrime';

    EVPrime = [zeros(N,6) diag(diag(ones(N,N)))];
    
    SparsityPattern = (sum1 + sum2 - EVPrime);
    
    size(SparsityPattern)

    SparsityPattern = ceil(abs(SparsityPattern))./max(ceil(abs(SparsityPattern)),1);
    

%END Prepare the sparsity pattern of constraints gradient  
    
% ktropts = optimset('Display','iter',...
%           'GradConstr','on','GradObj','on','TolCon',1E-15,'TolFun',1E-15,'TolX',1E-15,'JacobPattern',SparsityPattern);

      
ktropts = optimset('Display','iter',...
          'GradConstr','on','GradObj','on','TolCon',1E-6,'TolFun',1E-6,'TolX',1E-6,'JacobPattern',SparsityPattern);
      
      
%Initial estimates
X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime;  
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%--------------------------------------------------------------------------
%---knitro with gradient and with gradient sparsity
%---Termination conditions adjusted to meet fmincon results
%--------------------------------------------------------------------------
ktropts = optimset('Display','iter',...
          'GradConstr','on','GradObj','on','TolCon',1E-6,'TolFun',1,'TolX',1E-6,'JacobPattern',SparsityPattern);

%Initial estimates      
X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime;  
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%--------------------------------------------------------------------------
%---knitro with gradient and without gradient sparsity
%--------------------------------------------------------------------------

   
ktropts = optimset('Display','iter',...
          'GradConstr','on','GradObj','on','TolCon',1E-6,'TolFun',1E-6,'TolX',1E-6);

%Initial estimates
X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime;  
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%--------------------------------------------------------------------------
%---knitro with gradient and without gradient sparsity
%---Termination conditions adjusted to meet fmincon results
%--------------------------------------------------------------------------
ktropts = optimset('Display','iter',...
          'GradConstr','on','GradObj','on','TolCon',1E-6,'TolFun',1,'TolX',1E-6);

%Initial estimates      
X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime; 
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%---------------------------------------------------------
%---fmincon without gradient
%---(fmincon does not take gradient sparsity into account)
%---fmincon stops before TolFun<1E-4
%---------------------------------------------------------

%Initial estimates
X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

fminconopts = optimset('Display','iter',...
          'GradConstr','off','GradObj','off','TolCon',1E-4,'TolFun',1E-4,'TolX',1E-4);

% tic
t = cputime;
[X fval flag] = fmincon(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,fminconopts);
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%--------------------------------------------------------------------------
%---knitro without gradient and without gradient sparsity
%--------------------------------------------------------------------------

ktropts = optimset('Display','iter',...
          'GradConstr','off','GradObj','off','TolCon',1E-6,'TolFun',1E-6,'TolX',1E-6);

X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime;
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%--------------------------------------------------------------------------
%---knitro without first-order gradients and with gradient sparsity
%---(for comparison purpose with AMPL knitro)
%--------------------------------------------------------------------------

ktropts = optimset('Display','iter',...
          'GradConstr','off','GradObj','off','TolCon',1E-6,'TolFun',1E-6,'TolX',1E-6,'JacobPattern',SparsityPattern);

X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime;  
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];
%--------------------------------------------------------------------------
%---knitro without gradient and without gradient sparsity
%---Termination conditions adjusted to meet fmincon results
%--------------------------------------------------------------------------

ktropts = optimset('Display','iter',...
          'GradConstr','off','GradObj','off','TolCon',1E-6,'TolFun',1,'TolX',1E-6);

X(7:(7-1+N))=0;
X(6)=0;
X(1)=0;
X(2)=0;
X(3:5)=1/3;

% tic
t = cputime;  
[X fval flag] = ktrlink(@likelihood,X,[],[],Aeq,beq,lb,ub,@confuneq,ktropts);      
tsol = cputime - t;
% toc

Xsol = [Xsol, X'];
Tsol = [Tsol, tsol]; 
fvalsol = [fvalsol, fval];
flagsol = [flagsol, flag];

diary off;