function [f,g,h] = likelihood(X)
%Likelihood of the HAROLD ZURCHER BUS REPAIR EXAMPLE

global dt xt PayoffDiffPrime TransProbPrime

[CbEV, EV, TransProb, thetaProbs, RC] = DefVar(X);

%  Let PayoffDiff[i] represent -CbEV[i] - RC + CbEV[1]; 
%  this is the difference in expected payoff at x[i] between engine replacement and regular maintenance
PayoffDiff  = -CbEV - RC + CbEV(1);               
%  Let ProbRegMaint[i] represent 1/(1+exp(PayoffDiff[i])); 
%  this is the probability of performing regular maintenance at state x[i];
ProbRegMaint = 1./(1+exp(PayoffDiff)); 

%#
%#####  OBJECTIVE AND CONSTRAINT DEFINITIONS  #####
%#
%## Define objective function
%##
%## # Likelihood function 

% The likelihood function contains two pieces
% First is the likelihood that the engine is replaced given time t state in the data.
% Second is the likelihood that the observed transition between t-1 and t
% would have occurred.
nT = length(dt);
dtM1Minus = find(dt(1:(nT-1))==0);
dtM1Plus = find(dt(1:(nT-1))==1);

dtMinus = find(dt(2:nT)==0)+1;
dtPlus = find(dt(2:nT)==1)+1;

% 

f = -( sum( log( 1-ProbRegMaint(xt(dtPlus))))...
             + sum( log( ProbRegMaint(xt(dtMinus)))) ...
             +  sum( log( TransProb( xt( dtM1Plus +1 ) ) ))...
             + sum( log( TransProb(xt(dtM1Minus+1)-xt(dtM1Minus)+1))) ); 
    
    if nargout > 1
                
        d1 = PayoffDiffPrime(:,xt(dtPlus))*ProbRegMaint(xt(dtPlus)) ;
        
        d2 = - PayoffDiffPrime(:,xt(dtMinus))*( 1-ProbRegMaint(xt(dtMinus)) );
               
        d3 = TransProbPrime(:, xt( dtM1Plus +1 ))*(1./TransProb( xt( dtM1Plus +1 ) ));
        
        d4 = TransProbPrime(:, xt(dtM1Minus+1)-xt(dtM1Minus)+1)*(1./TransProb( xt(dtM1Minus+1)-xt(dtM1Minus)+1 ));
                
        g=-(d1+d2+d3+d4);        
    end
    if nargout > 2  
        h=[];
    end
    
end

